@@ -1,4 +1,4 @@
-# Copyright (C) 2010 The Backup Manager Authors
+# Copyright © 2005-2016 The Backup Manager Authors
 #
 # See the AUTHORS file for details.
 #
@@ -38,19 +38,18 @@ function commit_archive()
         echo "$str ${md5hash})"
     fi
 
-    md5file="$BM_REPOSITORY_ROOT/${BM_ARCHIVE_PREFIX}-${TODAY}.md5"
-
     # Check if the md5file contains already the md5sum of the file_to_create.
     # In this case, the new md5sum overwrites the old one.
-    if grep "$base" $md5file >/dev/null 2>&1 ; then
-        previous_md5sum=$(get_md5sum_from_file $base $md5file)
-        sed -e "/$base/s/$previous_md5sum/$md5hash/" -i $md5file
+    if grep "$base" $MD5FILE >/dev/null 2>&1 ; then
+        previous_md5sum=$(get_md5sum_from_file $base $MD5FILE)
+        sed -e "/$base/s/$previous_md5sum/$md5hash/" -i $MD5FILE
     else
-        echo "$md5hash  $base" >> $md5file
+        echo "$md5hash  $base" >> $MD5FILE
     fi
 
-    # Now that the file is created, remove previous duplicates if exists...
-    purge_duplicate_archives $file_to_create || 
+    # Now that the file is created, remove previous duplicates if exist.
+    # Pass in the calculated (costly) md5 sum, to speed things up.
+    purge_duplicate_archives $file_to_create $md5hash ||
         error "Unable to purge duplicates of \$file_to_create"
 
     # ownership
@@ -167,10 +166,13 @@ function __exec_meta_command()
                 tail_logfile "$logfile"
                 if [[ "$BM_ENCRYPTION_METHOD" = "gpg" ]]; then
                     $command 2>$logfile | $nice $compress_bin -f -q -9 2>$logfile | $nice $gpg $BM__GPG_HOMEDIR -r "$BM_ENCRYPTION_RECIPIENT" -e > $file_to_create.$ext.gpg 2> $logfile
+                    cmdpipestatus=${PIPESTATUS[0]}
                     debug "$command | $nice $compress_bin -f -q -9 | $nice $gpg $BM__GPG_HOMEDIR -r \"$BM_ENCRYPTION_RECIPIENT\" -e > $file_to_create.$ext.gpg 2> $logfile"
                     file_to_create="$file_to_create.$ext.gpg"
                 else
                     $command 2> $logfile | $nice $compress_bin -f -q -9 > $file_to_create.$ext 2> $logfile
+                    cmdpipestatus=${PIPESTATUS[0]}
+                    debug "$command 2> $logfile | $nice $compress_bin -f -q -9 > $file_to_create.$ext 2> $logfile"
                     file_to_create="$file_to_create.$ext"
                 fi
 
@@ -178,7 +180,12 @@ function __exec_meta_command()
                     warning "Unable to exec \$command; check \$logfile"
                     rm -f $file_to_create
                 else
-                    rm -f $logfile
+                    if [[ $cmdpipestatus -gt 0 ]]; then
+                        warning "Unable to exec first piped command \$command; check \$logfile"
+                        rm -f $file_to_create
+                    else
+                        rm -f $logfile
+                    fi
                 fi
             else
                 error "Compressor \$compress is needed."
@@ -239,7 +246,9 @@ function __get_flags_relative_blacklist()
     target="$2"
     debug "__get_flags_relative_blacklist ($switch, $target)"
 
-    target=${target%/}
+    if [ "$target" != "/" ]; then
+        target=${target%/}
+    fi
     blacklist=""
     for pattern in $BM_TARBALL_BLACKLIST
     do
@@ -253,7 +262,13 @@ function __get_flags_relative_blacklist()
                 # making a relative path...
                 pattern="${pattern#$target}"
                 length=$(expr length $pattern)
-                pattern=$(expr substr $pattern 2 $length)
+                # for $target="/", no spare / is left at the beggining
+                # after the # substitution; thus take substr from pos 1
+                if [ "$target" != "/" ]; then
+                    pattern=$(expr substr $pattern 2 $length)
+                else
+                    pattern=$(expr substr $pattern 1 $length)
+                fi
 
                 # ...and blacklisting it
                 blacklist="$blacklist ${switch}${pattern}"
@@ -782,7 +797,7 @@ function __make_local_tarball_token
             "dar")
                 __get_flags_dar_incremental "$dir_name"
             ;;
-            "tar"|"tar.gz"|"tar.bz2")
+            "tar"|"tar.gz"|"tar.bz2"|"tar.lz")
                 __get_flags_tar_incremental "$dir_name"
             ;;
             esac
@@ -879,26 +894,43 @@ function backup_method_pgsql()
     debug "backup_method_pgsql ($method)"
 
     info "Using method \"\$method\"."
-    if [[ ! -x $pgdump ]]; then
-        error "The \"pgsql\" method is chosen, but \$pgdump is not found."
+    if [[ -x $pgdump ]] && [[ -x ${pgdump}all ]]; then
+        :
+    else
+        error "The \"postgresql\" method is chosen, but \$pgdump and/or \$pgdumpall are not found."
     fi
 
-    opt=" -U$BM_PGSQL_ADMINLOGIN -h$BM_PGSQL_HOST -p$BM_PGSQL_PORT" 
+    # Allow empty host when connecting to postgress with unix sockets.
+
+    if [[ "X$BM_PGSQL_HOST" = "X" ]]; then
+        BM_PGSQL_HOSTFLAGS=""
+    else
+        BM_PGSQL_HOSTFLAGS="-h$BM_PGSQL_HOST"
+    fi
+    opt=" -U$BM_PGSQL_ADMINLOGIN $BM_PGSQL_HOSTFLAGS -p$BM_PGSQL_PORT"
+
+    # We need a second variable, to know if the backup pgpass file was used.
+
     BM_SHOULD_PURGE_PGPASS="false"
+    BM_USING_BACKUP_PGPASS="false"
 
     if [[ -f $pgsql_conffile ]]; then
         info "Found existing PgSQL client configuration file: \$pgsql_conffile"
         info "Looking for matching credentials in this file..."
         if ! grep -qE "(${BM_PGSQL_HOST}|[^:]*):(${BM_PGSQL_PORT}|[^:]*):[^:]*:${BM_PGSQL_ADMINLOGIN}:${BM_PGSQL_ADMINPASS}" $pgsql_conffile; then
             info "No matching credentials: inserting our own."
-            cp $pgsql_conffile $pgsql_conffile_bm
             BM_SHOULD_PURGE_PGPASS="true"
-            echo "${BM_PGSQL_HOST}:${BM_PGSQL_PORT}:${BM_PGSQL_ADMINLOGIN}:${BM_PGSQL_ADMINPASS}" >> $pgsql_conffile
+            BM_USING_BACKUP_PGPASS="true"
+            mv $pgsql_conffile $pgsql_conffile_bm
+            touch $pgsql_conffile
+            chmod 0600 $pgsql_conffile
+            echo "${BM_PGSQL_HOST}:${BM_PGSQL_PORT}:*:${BM_PGSQL_ADMINLOGIN}:${BM_PGSQL_ADMINPASS}" >> $pgsql_conffile
         fi
     else
         warning "Creating a default PgSQL client configuration file: \$HOME/.pgpass"
-        echo "${BM_PGSQL_HOST}:${BM_PGSQL_PORT}:${BM_PGSQL_ADMINLOGIN}:${BM_PGSQL_ADMINPASS}" >> $pgsql_conffile
+        touch $pgsql_conffile
         chmod 0600 $pgsql_conffile
+        echo "${BM_PGSQL_HOST}:${BM_PGSQL_PORT}:*:${BM_PGSQL_ADMINLOGIN}:${BM_PGSQL_ADMINPASS}" >> $pgsql_conffile
     fi
 
     compress="$BM_PGSQL_FILETYPE"
@@ -909,7 +941,7 @@ function backup_method_pgsql()
             file_to_create="$BM_REPOSITORY_ROOT/${BM_ARCHIVE_PREFIX}-all-pgsql-databases.$TODAY.sql"
             command="${pgdump}all $opt $BM_PGSQL_EXTRA_OPTIONS"
         else
-            file_to_create="$BM_REPOSITORY_ROOT/${BM_ARCHIVE_PREFIX}-${database}.$TODAY.sql"
+            file_to_create="$BM_REPOSITORY_ROOT/${BM_ARCHIVE_PREFIX}-pgsql-${database}.$TODAY.sql"
             command="$pgdump $opt $database $BM_PGSQL_EXTRA_OPTIONS"
         fi
         __create_file_with_meta_command
@@ -917,9 +949,13 @@ function backup_method_pgsql()
 
     # purge the .pgpass file, if created by Backup Manager
     if [[ "$BM_SHOULD_PURGE_PGPASS" == "true" ]]; then
-        info "restoring initial .pgpass file."
-        warning "To avoid problems with .pgpass, insert the configured host:port:user:pass in $pgsql_conffile"
-        mv $pgsql_conffile_bm $pgsql_conffile
+        info "Removing default PostgreSQL password file: \$pgsql_conffile"
+        rm -f $pgsql_conffile
+        if [[ "$BM_USING_BACKUP_PGPASS" == "true" ]]; then
+            info "restoring initial \$pgsql_conffile file from backup."
+            warning "To avoid problems with \$pgsql_conffile, insert the configured host:port:database:user:password inside."
+            mv $pgsql_conffile_bm $pgsql_conffile
+        fi
     fi
 }
 
@@ -957,13 +993,32 @@ function backup_method_mysql()
     base_command="$mysqldump --defaults-extra-file=$mysql_conffile $opt -u$BM_MYSQL_ADMINLOGIN -h$BM_MYSQL_HOST -P$BM_MYSQL_PORT $BM_MYSQL_EXTRA_OPTIONS"
     compress="$BM_MYSQL_FILETYPE"   
 
+    # get each DB name if backing up separately
+    if [ "$BM_MYSQL_DATABASES" = "__ALL__" ]; then
+        if [ "$BM_MYSQL_SEPARATELY" = "true" ]; then
+            if [[ ! -x $mysql ]]; then
+                error "Can't find "$mysql" but this is needed when backing up databases separately."
+            fi
+
+            DBNAMES=$($mysql --defaults-extra-file=$mysql_conffile -u $BM_MYSQL_ADMINLOGIN -h $BM_MYSQL_HOST -P $BM_MYSQL_PORT -B -N -e "show databases" | sed 's/ /%/g')
+
+            # if DBs are excluded
+            for exclude in $BM_MYSQL_DBEXCLUDE
+            do
+                DBNAMES=$(echo $DBNAMES | sed "s/\b$exclude\b//g")
+            done
+
+            BM_MYSQL_DATABASES=$DBNAMES
+        fi
+    fi
+
     for database in $BM_MYSQL_DATABASES
     do
         if [[ "$database" = "__ALL__" ]]; then
             file_to_create="$BM_REPOSITORY_ROOT/${BM_ARCHIVE_PREFIX}-all-mysql-databases.$TODAY.sql"
             command="$base_command --all-databases"
         else
-            file_to_create="$BM_REPOSITORY_ROOT/${BM_ARCHIVE_PREFIX}-${database}.$TODAY.sql"
+            file_to_create="$BM_REPOSITORY_ROOT/${BM_ARCHIVE_PREFIX}-mysql-${database}.$TODAY.sql"
             command="$base_command $database"
         fi
         __create_file_with_meta_command
